/***************************************************************************
Name: CS Lite
Description: Control cookie permissions.
Author: wantora
Homepage: https://bitbucket.org/wantora/cslite-mod/

Copyright (C) 2007  Ron Beckman
Copyright (C) 2011  wantora

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");


const CSLAST10HOSTS_CONTRACTID = '@wantora.bitbucket.org/cookiesafe-last10hosts;1';
const CSLAST10HOSTS_CID = Components.ID('{d4295e7f-ac82-47d5-ab40-a3781cb49980}');
const CSLAST10HOSTS_IID = Components.interfaces.nsICookieSafeLast10Hosts;
const CSLAST10HOSTS_SERVICENAME = 'CS Last 10 Hosts';

function nsCSLast10Hosts(){}
nsCSLast10Hosts.prototype = {
	classDescription: CSLAST10HOSTS_SERVICENAME,
	classID: CSLAST10HOSTS_CID,
	contractID: CSLAST10HOSTS_CONTRACTID,

	hosts: [],

	testLastTenHosts: function(host) {
		for (var i=0; i<this.hosts.length; ++i) {
			if (this.hosts[i]==host) return true;
		}

		return false;
	},

	getLastTenHosts: function() {
		return this.hosts.join(' ');
	},

	addLastTenHosts: function(host) {
		var found = this.testLastTenHosts(host);
		if (!found) {
			this.hosts.unshift(host);
		}

		//make sure the hosts array length does not exceed the numOfHosts pref
		var prefs = this.getPrefs();
		var num = parseInt(prefs.getIntPref('numOfHosts'));
		while (this.hosts.length > num) {
			this.hosts.pop();
		}
	},

	removeLastTenHosts: function(host) {
		this.hosts = this.hosts.filter(function(value) {
								return value != host; 
							 });
	},

	clearLastTenHosts: function() {
		this.hosts = [];
	},

	getPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("extensions.cookiesafe.");
	},

	QueryInterface: XPCOMUtils.generateQI([CSLAST10HOSTS_IID])
};

/**
* XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
* XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
*/
if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory([nsCSLast10Hosts]);
else
    var NSGetModule = XPCOMUtils.generateNSGetModule([nsCSLast10Hosts]);
