/***************************************************************************
Name: CS Lite
Description: Control cookie permissions.
Author: wantora
Homepage: https://bitbucket.org/wantora/cslite-mod/

Copyright (C) 2007  Ron Beckman
Copyright (C) 2011  wantora

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");


const CSTEMPEXCEPTIONS_CONTRACTID = '@wantora.bitbucket.org/cookiesafe-temp-exceptions;1';
const CSTEMPEXCEPTIONS_CID = Components.ID('{8742629c-8f6a-4787-9c44-2db49b392531}');
const CSTEMPEXCEPTIONS_IID = Components.interfaces.nsICookieSafeTempExceptions;
const CSTEMPEXCEPTIONS_SERVICENAME = 'CS Temp Exceptions';

function nsCSTempExceptions(){}
nsCSTempExceptions.prototype = {
	classDescription: CSTEMPEXCEPTIONS_SERVICENAME,
	classID: CSTEMPEXCEPTIONS_CID,
	contractID: CSTEMPEXCEPTIONS_CONTRACTID,

	hosts: [],

	testTempExceptions: function(host) {
		for (var i=0; i<this.hosts.length; ++i) {
			if (this.hosts[i]==host) return true;
		}

		return false;
	},

	getTempExceptions: function() {
		return this.hosts.join(' ');
	},

	addTempExceptions: function(host) {
		var found = this.testTempExceptions(host);
		if (!found) {
			this.hosts.push(host);
		}

		//keep track of temp exceptions in a char pref in case browser
		//crashes before all of the temp exceptions have been cleared
		var prefs = this.getPrefs();
		prefs.setCharPref('tempExceptions',this.hosts.join(' '));
	},

	removeTempExceptions: function(host) {
		this.hosts = this.hosts.filter(function(value) {
								return value != host; 
							 });

		//keep track of temp exceptions in a char pref in case browser
		//crashes before all of the temp exceptions have been cleared
		var prefs = this.getPrefs();
		prefs.setCharPref('tempExceptions',this.hosts.join(' '));
	},

	clearTempExceptions: function() {
		this.hosts = [];

		//keep track of temp exceptions in a char pref in case browser
		//crashes before all of the temp exceptions have been cleared
		var prefs = this.getPrefs();
		prefs.setCharPref('tempExceptions','');
	},

	getPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("extensions.cookiesafe.");
	},

	QueryInterface: XPCOMUtils.generateQI([CSTEMPEXCEPTIONS_IID])
};

/**
* XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
* XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
*/
if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory([nsCSTempExceptions]);
else
    var NSGetModule = XPCOMUtils.generateNSGetModule([nsCSTempExceptions]);
